//
//  EWETextField.m
//  Demo
//
//  Created by Niku on 16/9/13.
//  Copyright © 2016年 Ewell. All rights reserved.
//

#import "EWETextField.h"
#import "EWEDefTextFieldLimitation.h"

@implementation UITextField(properySupport)

- (void)setPropertyByTextType:(int)type {
    switch (type) {
        case TEXTTYPE_NUMBER:
        case TEXTTYPE_MONEY:
        case TEXTTYPE_RATIO:
        case TEXTTYPE_INTEGER:
        case TEXTTYPE_DECIMAL:
        case TEXTTYPE_DATE:
            self.textAlignment = NSTextAlignmentRight;
            self.keyboardType = UIKeyboardTypeNumberPad;
            break;
            
        case TEXTTYPE_TELPST_NUM:
        case TEXTTYPE_ALPHABET:
        case TEXTTYPE_ALPHANUMELIC:
        case TEXTTYPE_ALPHANUMELICSYMBOL:
            self.keyboardType = UIKeyboardTypeASCIICapable;
            break;
            
        case TEXTTYPE_URL:
            self.keyboardType = UIKeyboardTypeURL;
            break;
            
        case TEXTTYPE_E_MAIL:
            self.keyboardType = UIKeyboardTypeEmailAddress;
            break;
            
        default:
            break;
    }
}

@end
@implementation EWETextField
@synthesize tvCheckHelper;

- (id)initWithFrame:(CGRect)frame
{
    self = [super initWithFrame:frame];
    if (self) {
        self.borderStyle = UITextBorderStyleRoundedRect;
        self.clearButtonMode = UITextFieldViewModeWhileEditing;
        tvCheckHelper = [[TextValueCheckHelper alloc] init];
        self.delegate = tvCheckHelper;
        self.contentVerticalAlignment = UIControlContentVerticalAlignmentCenter;
        self.returnKeyType = UIReturnKeyNext;
        [[NSNotificationCenter defaultCenter]addObserver:tvCheckHelper selector:@selector(textFiledEditChanged:)
                                                    name:@"UITextFieldTextDidChangeNotification"
                                                  object:self];
    }
    return self;
}

- (void)awakeFromNib {
    [super awakeFromNib];
    self.borderStyle = UITextBorderStyleRoundedRect;
    self.clearButtonMode = UITextFieldViewModeWhileEditing;
    if (self.delegate == nil) {
        tvCheckHelper = [[TextValueCheckHelper alloc] init];
        self.delegate = tvCheckHelper;
        [[NSNotificationCenter defaultCenter]addObserver:tvCheckHelper selector:@selector(textFiledEditChanged:)
                                                                                  name:@"UITextFieldTextDidChangeNotification"
                                                                                object:self];
    }
    self.returnKeyType = UIReturnKeyNext;
}

- (void)dealloc{
    [[NSNotificationCenter defaultCenter] removeObserver:tvCheckHelper name:@"UITextFieldTextDidChangeNotification" object:self];
}


/**
	设定限定
 右对齐  以及设置相对的键盘
	@param type
 */
- (void)setCheckingType:(int)type{
    tvCheckHelper.checkingType = type;
    [self setPropertyByTextType:type];
}

- (void)drawPlaceholderInRect:(CGRect)rect {
//    [RGB(136,173,166) setFill];

    UIFont *font = [UIFont systemFontOfSize:16];
    
    NSMutableParagraphStyle *style = [[NSMutableParagraphStyle alloc] init];
    style.lineBreakMode = NSLineBreakByCharWrapping;
    style.alignment = NSTextAlignmentLeft;
    NSDictionary *attributes = @{
                                 NSForegroundColorAttributeName : [UIColor whiteColor],
                                 NSFontAttributeName : font,
                                 NSParagraphStyleAttributeName : style
                                 };
    //
    if (self.attributedPlaceholder) {
        [[self placeholder] drawInRect:rect withAttributes:[self.attributedPlaceholder attributesAtIndex:0 effectiveRange:nil]];
    }else{
        [[self placeholder] drawInRect:rect withAttributes:attributes];
    }
}

- (NSString *)getFormatedText {
    if (self.text == nil) {
        return self.text;
    }
    return [tvCheckHelper format:self.text formatType:tvCheckHelper.formatType];
}

/*
// Only override drawRect: if you perform custom drawing.
// An empty implementation adversely affects performance during animation.
- (void)drawRect:(CGRect)rect {
    // Drawing code
}
*/

@end
