//
//  TextValueCheckHelper.h
//  Demo
//
//  Created by Niku on 16/9/13.
//  Copyright © 2016年 Ewell. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "UIKit/UIKit.h"

@interface TextValueCheckHelper : NSObject<UITextFieldDelegate>
/*! 类型 */
@property int checkingType;

/*! format种类 */
@property int formatType;

// 输入辅助用formatType来判断

/*! 数値的范围。最小値。转换成float之后处理 */
@property (retain,nonatomic) NSNumber* min;

/*! 数値的范围。最大値。转换成float之后处理 */
@property (retain,nonatomic) NSNumber* max;

/*! NG的数値。内容はNSNumber。转换成float之后处理 */
@property (retain,nonatomic) NSArray* ngArray;

/*! 日期范围。开始日期。 */
@property (retain,nonatomic) NSDate* earlyDay;

/*! 日期范围。结束日期。 */
@property (retain,nonatomic) NSDate* laterDay;

/*! 値和时间之外、警告颜色 */
@property (retain,nonatomic) UIColor* ngColor;

/*! 値和时间之外、正常颜色  */
@property (retain,nonatomic) UIColor* okColor;

@property int maxlength;


/*!
 返回是否显示输入值
 @param text pattern 能调用的文字列
 @param textType 应该输入的类型
 @returns YES:显示 NO:无视输入的值
 */
- (BOOL)checkLimitation:(NSString*)text limitationType:(int)textType;

/*!
 执行text的format
 @param text　需要被转换的文字
 @param formatType 转换后的类型
 @returns 转换后的文字
 */
- (NSString*)format:(NSString*)text formatType:(int)type;
// 只能指定一种类型

/*!
 输入辅助
 @param text textfield里现有的值
 @param deformatType 转换前的类型
 @returns 转换前的文字
 */
- (NSString*)deformat:(NSString*)text deformatType:(int)type;


/*!
 判断是否是正常的值
 */
/*!
 数值，日期是否正常在被决定的范围之内
 数値や日付が正常かつ決められた範囲に収まっているか、<br>および何らかのコード値がマスタと一致するかを判定。
 @param tf 输入的值
 @param type 期望的类型
 @returns    YES:规定内的值
 NO:规定外的值  （错误的话   红色表示）
 */
- (BOOL)isNormalValue:(UITextField*)tf formatType:(int)type;


/*!
 指定数字型数据范围
 @param min 最小値
 @param max 最大値
 */
- (void)setValueRangeMinToMax:(NSNumber*)min max:(NSNumber*)max;

/*!
 指定日期型数据范围
 @param early    开始日期
 @param late     结束日期
 */
- (void)setDateRangeWithEarly:(NSDate*)early late:(NSDate*)late;


- (void)textFiledEditChanged:(NSNotification*)obj;

@end

@protocol TextValueCheckDelegate <NSObject>

- (void)didReadBarcodePrefix;

@end
