//
//  TextValueCheckHelper.m
//  Demo
//
//  Created by Niku on 16/9/13.
//  Copyright © 2016年 Ewell. All rights reserved.
//

#ifdef PUT
#define PUT_LOG(...) NSLog(__VA_ARGS__)
#define PUT_LOG_METHOD NSLog(@"%s", __func__)
#else
#define PUT_LOG(...) ;
#define PUT_LOG_METHOD ;
#endif

#import "TextValueCheckHelper.h"
#import "EWEDefTextFieldLimitation.h"

#define RegInt      @"0|[1-9][0-9]*"
#define RegAlp      @"[A-Za-zＡＢＣＤＥＦＧＨＩＪＫＬＭＮＯＰＱＲＳＴＵＶＷＸＹＺａｂｃｄｅｆｇｈｉｊｋｌｍｎｏｐｑｒｓｔｕｖｗｘｙｚ]*"
#define RegAlpNum   @"[A-Za-zＡＢＣＤＥＦＧＨＩＪＫＬＭＮＯＰＱＲＳＴＵＶＷＸＹＺａｂｃｄｅｆｇｈｉｊｋｌｍｎｏｐｑｒｓｔｕｖｗｘｙｚ0-9０-９]*"
#define RegAlpNumSymbol   @"[A-Za-z0-9+\\-*#\\@/_=;:\\[\\]\\(\\)]*"
#define RegDec      @"[0-9]*\\.[0-9]*"
#define RegMoney    @"0|[1-9][0-9]*"
//#define RegMoney    @"(\\$|\\¥)?(0|[1-9][0-9]*)?"
#define RegNumInt   @"^-(0|[1-9][0-9]*)?"
#define RegNumDec   @"-[0-9]*\\.?[0-9]*"
// 参考 oshiete.goo.ne.jp/qa/1418896.html
//     ameblo.jp/tondemonight/entry-10117100089
//     认为URL和Email地址输入校验一致是没有问题的。
#define RegURL      @"[-@a-zA-Z0-9ＡＢＣＤＥＦＧＨＩＪＫＬＭＮＯＰＱＲＳＴＵＶＷＸＹＺａｂｃｄｅｆｇｈｉｊｋｌｍｎｏｐｑｒｓｔｕｖｗｘｙｚ０-９,'''\\._!#\\$`\\*\\+/\\?\\^\\{\\}\\|~&%=\\(\\)<>\\[\\]¥:;\"]*"
#define RegEmail    @"[-@a-zA-Z0-9ＡＢＣＤＥＦＧＨＩＪＫＬＭＮＯＰＱＲＳＴＵＶＷＸＹＺａｂｃｄｅｆｇｈｉｊｋｌｍｎｏｐｑｒｓｔｕｖｗｘｙｚ０-９,'''\\._!#\\$`\\*\\+/\\?\\^\\{\\}\\|~&%=\\(\\)<>\\[\\]¥:;\"]*"
//#define RegDate     @"[0-9]{0,4}-?[0-9]{0,2}?-?[0-9]{0,2}? | [0-9]{0,4}?/?[0-9]{0,2}?/?[0-9]{0,2}?"
#define RegDate     @"^[0-9][-/0-9]*"
#define RegNumArr   @"[0-9]*"
#define RegTelPst   @"^[0-9][-0-9]*"
#define RegRatio    @"[0-9%]*\\.[0-9]?[0-9]?[0-9]?[0-9]?"
#define RegFullKana @"^[ぁ-んァ-ヶーゝゞ゛゜]+$"
#define CLASS_NAME  @"textValueCheckHelper"
//#define RegUser
#define CheckTypeCombileNo -1

@implementation TextValueCheckHelper

@synthesize checkingType;
@synthesize formatType;
@synthesize min = _min;
@synthesize max = _max;
@synthesize ngArray = _ngArray;
@synthesize earlyDay;
@synthesize laterDay;
@synthesize ngColor;
@synthesize okColor;
@synthesize maxlength;

- (id)init{
    self = [super init];
    if(self){
        ngColor = [UIColor redColor];
        okColor = [UIColor blackColor];
    }
    return self;
}


/*****************************************************************************
**
 返回是否显示输入值
 @param text pattern パターンと一致しているかを調べられる文字列
 @param textType 应该输入的类型
 @returns YES:显示 NO:无视输入的值
 */
- (BOOL)checkLimitation:(NSString *)text limitationType:(int)textType
{
    PUT_LOG(@"matchType = %d",[self matchPattern:text]);
    if((textType == TEXTTYPE_DECIMAL) ||
       (textType == TEXTTYPE_RATIO)){
        if([text rangeOfString:@"."].location != INT32_MAX &&
           ((text.length-[text rangeOfString:@"."].location)-1 > 4)){
            return NO;
        }
    }
    
    return [self isCorrect:[self matchPattern:text chekingType:textType] chekingType:textType];
}

/**
 判断文字列适合何种正规表现
 @param text 需要判断的文字
 @returns 适用的类型
 */
- (int)matchPattern:(NSString*)text chekingType:(int)textType{
    
    PUT_LOG_METHOD;
    
    int textMatchType = TEXTTYPE_NONE;
    
    /** check */
    NSRange range;
    /** 小数 */
    range = [text rangeOfString:RegDec options:NSRegularExpressionSearch];
    if ( [text length] == range.length ) {
        textMatchType = TEXTTYPE_DECIMAL;
        if (textType == textMatchType) {
            return textMatchType;
        }
    }
    /** 整数 */
    range = [text rangeOfString:RegInt options:NSRegularExpressionSearch];
    if ( [text length] == range.length ) {
        textMatchType = TEXTTYPE_INTEGER;
        if (textType == textMatchType) {
            return textMatchType;
        }
    }
    /** 英字 */
    range = [text rangeOfString:RegAlp options:NSRegularExpressionSearch];
    if ( [text length] == range.length ) {
        textMatchType = TEXTTYPE_ALPHABET;
        if (textType == textMatchType) {
            return textMatchType;
        }
    }
    /** 数字列 */
    range = [text rangeOfString:RegNumArr options:NSRegularExpressionSearch];
    if ( [text length] == range.length ) {
        textMatchType = TEXTTYPE_NUMBER_ARR;
        if (textType == textMatchType) {
            return textMatchType;
        }
    }
    /** 英数字 */
    range = [text rangeOfString:RegAlpNum options:NSRegularExpressionSearch];
    if ( [text length] == range.length ) {
        textMatchType = TEXTTYPE_ALPHANUMELIC;
        if (textType == textMatchType) {
            return textMatchType;
        }
    }
    /** 货币 */
    range = [text rangeOfString:RegMoney options:NSRegularExpressionSearch];
    if ( [text length] == range.length ) {
        textMatchType = TEXTTYPE_MONEY;
        if (textType == textMatchType) {
            return textMatchType;
        }
    }
    /** 带符号的整数 */
    range = [text rangeOfString:RegNumInt options:NSRegularExpressionSearch];
    if ( [text length] == range.length ) {
        textMatchType = TEXTTYPE_NUMBER_INT;
        if (textType == textMatchType) {
            return textMatchType;
        }
    }
    /**　带符号的小数 */
    range = [text rangeOfString:RegNumDec options:NSRegularExpressionSearch];
    if ( [text length] == range.length ) {
        textMatchType = TEXTTYPE_NUMBER_DEC;
        if (textType == textMatchType) {
            return textMatchType;
        }
    }
    /** 日期 */
    range = [text rangeOfString:RegDate options:NSRegularExpressionSearch];
    if ( [text length] == range.length ) {
        textMatchType = TEXTTYPE_DATE;
        if (textType == textMatchType) {
            return textMatchType;
        }
    }
    /** 比率 */
    range = [text rangeOfString:RegRatio options:NSRegularExpressionSearch];
    if ( [text length] == range.length ) {
        textMatchType = TEXTTYPE_RATIO;
        if (textType == textMatchType) {
            return textMatchType;
        }
    }
    /** e-mail */
    range = [text rangeOfString:RegEmail options:NSRegularExpressionSearch];
    if ( [text length] == range.length ) {
        textMatchType = TEXTTYPE_E_MAIL;
        if (textType == textMatchType) {
            return textMatchType;
        }
    }
    /** URL */
    range = [text rangeOfString:RegURL options:NSRegularExpressionSearch];
    if ( [text length] == range.length ) {
        textMatchType = TEXTTYPE_URL;
        if (textType == textMatchType) {
            return textMatchType;
        }
    }
    /** TEL */
    range = [text rangeOfString:RegTelPst options:NSRegularExpressionSearch];
    if ( [text length] == range.length ) {
        textMatchType = TEXTTYPE_TELPST_NUM;
        if (textType == textMatchType) {
            return textMatchType;
        }
    }
//    /** 平假名 */
//    range = [text rangeOfString:RegFullKana options:NSRegularExpressionSearch];
//    if ( [text length] == range.length ) {
//        textMatchType = TEXTTYPE_KANA;
//        if (textType == textMatchType) {
//            return textMatchType;
//        }
//    }
    /** 英数字（带记号） */
    range = [text rangeOfString:RegAlpNumSymbol options:NSRegularExpressionSearch];
    if ( [text length] == range.length ) {
        textMatchType = TEXTTYPE_ALPHANUMELICSYMBOL;
        if (textType == textMatchType) {
            return textMatchType;
        }
    }
    
    return textMatchType;
}

/**
 判断输入的值是否符合类型
 @param type　适用的正规表现
 @returns YES:反映 NO:無視
 */
- (BOOL)isCorrect:(int)matchType chekingType:(int)textType{
    PUT_LOG_METHOD;
    PUT_LOG(@"%@ isCorrect matchType = %d",CLASS_NAME,matchType);
    PUT_LOG(@"%@ isCorrect textType = %d",CLASS_NAME,textType);
    
    switch (textType) {
        case TEXTTYPE_NONE:
            return YES;
            break;
            
        case TEXTTYPE_INTEGER:
            if(matchType == TEXTTYPE_INTEGER){
                return YES;
            }
            break;
            
        case TEXTTYPE_DECIMAL:{
            if((matchType == TEXTTYPE_INTEGER) || (matchType == TEXTTYPE_DECIMAL)){
                return YES;
            }
        }
            break;
            
        case TEXTTYPE_MONEY:
            //            if((matchType == TEXTTYPE_INTEGER) || (matchType == TEXTTYPE_MONEY)){
            //                return YES;
            //            }
            if((matchType == TEXTTYPE_INTEGER) || (matchType == TEXTTYPE_DECIMAL) ||
               (matchType == TEXTTYPE_NUMBER_INT) || (matchType == TEXTTYPE_NUMBER_DEC)){
                return YES;
            }
            
            break;
            
        case TEXTTYPE_ALPHABET:
            if(matchType == TEXTTYPE_ALPHABET){
                return YES;
            }
            break;
            
        case TEXTTYPE_ALPHANUMELIC:
            if((matchType == TEXTTYPE_ALPHABET) ||
               (matchType == TEXTTYPE_INTEGER) ||
               (matchType == TEXTTYPE_ALPHANUMELIC) ||
               (matchType == TEXTTYPE_NUMBER_ARR)){
                return YES;
            }
            break;
            
        case TEXTTYPE_NUMBER:
            if((matchType == TEXTTYPE_INTEGER) || (matchType == TEXTTYPE_NUMBER_ARR)){
                return YES;
            }
            break;
            
        case TEXTTYPE_URL:
            if((matchType == TEXTTYPE_URL) || (matchType == TEXTTYPE_E_MAIL) ||
               (matchType == TEXTTYPE_NUMBER_ARR) ||
               (matchType == TEXTTYPE_ALPHABET) ||
               (matchType == TEXTTYPE_ALPHANUMELIC) ||
               (matchType == TEXTTYPE_E_MAIL) ||
               (matchType == TEXTTYPE_INTEGER) || (matchType == TEXTTYPE_DECIMAL) ||
               (matchType == TEXTTYPE_NUMBER_INT) || (matchType == TEXTTYPE_NUMBER_DEC)){
                return YES;
            }
            break;
            
        case TEXTTYPE_E_MAIL:
            if((matchType == TEXTTYPE_URL) || (matchType == TEXTTYPE_E_MAIL) ||
               (matchType == TEXTTYPE_NUMBER_ARR) ||
               (matchType == TEXTTYPE_ALPHABET) ||
               (matchType == TEXTTYPE_ALPHANUMELIC) ||
               (matchType == TEXTTYPE_INTEGER) || (matchType == TEXTTYPE_DECIMAL) ||
               (matchType == TEXTTYPE_NUMBER_INT) || (matchType == TEXTTYPE_NUMBER_DEC)){
                return YES;
            }
            break;
            
        case TEXTTYPE_DATE:
            if((matchType == TEXTTYPE_DATE) ||
               (matchType == TEXTTYPE_INTEGER) ||
               (matchType == TEXTTYPE_NUMBER_ARR) ){
                return YES;
            }
            break;
            
        case TEXTTYPE_RATIO:
            if((matchType == TEXTTYPE_RATIO) ||
               (matchType == TEXTTYPE_INTEGER) || (matchType == TEXTTYPE_DECIMAL) ||
               (matchType == TEXTTYPE_NUMBER_INT) || (matchType == TEXTTYPE_NUMBER_DEC)){
                return YES;
            }
            break;
            
        case TEXTTYPE_TELPST_NUM:
            if (matchType == TEXTTYPE_DATE || matchType == TEXTTYPE_INTEGER ||
                matchType == TEXTTYPE_NUMBER_ARR || matchType == TEXTTYPE_TELPST_NUM) {
                return YES;
            }
            break;
            
//        case TEXTTYPE_KANA:
//            if (matchType == TEXTTYPE_KANA) {
//                return YES;
//            }
//            break;
            
        case TEXTTYPE_ALPHANUMELICSYMBOL:
            if(matchType == TEXTTYPE_ALPHANUMELICSYMBOL || matchType == TEXTTYPE_ALPHABET
               || matchType == TEXTTYPE_ALPHANUMELIC || matchType == TEXTTYPE_INTEGER
               || matchType == TEXTTYPE_NUMBER || matchType == TEXTTYPE_NUMBER_INT
               || matchType == TEXTTYPE_NUMBER_ARR || matchType == TEXTTYPE_TELPST_NUM){
                return YES;
            }
            break;
            
        default:
            
            break;
    }
    return NO;
}

/*****************************************************************************
**
 执行text的format
 @param text　需要被转换的文字
 @param formatType 转换后的类型
 @returns 转换后的文字
 */
- (NSString*)format:(NSString*)text formatType:(int)type
{
    if([text isEqualToString:@""]){
        return text;
    }
    
    NSString* returnString = text;
    switch (type) {
        case FORMAT_NONE:
            returnString = [self commaToFull:text];
            break;
        case FORMAT_NUM_COMMA:
            returnString = [self addComma:text];
            break;
        case FORMAT_ALP_TO_HALF:
            returnString = [self toHalf:text];
            break;
        case FORMAT_ALP_TO_FULL:
            returnString = [self toFull:text];
            break;
        case FORMAT_ALP_TO_UPPER:
            returnString = [text uppercaseString];
            break;
        case FORMAT_ALP_TO_LOWER:
            returnString = [text lowercaseString];
            break;
        case FORMAT_DATE_SLUSH:
            returnString = [self dateAddSlush:text];
            break;
        case FORMAT_DATE_KANJI:
            returnString = [self dateAddKanji:text];
            break;
        case FORMAT_RATIO:
            returnString = [self addPercent:text];
            break;
        case FORMAT_MONEY_INT_YEN:
            returnString = [self toMoneyIntYen:text];
            break;
        case FORMAT_MONEY_DEC_YEN:
            returnString = [self toMoneyDecYen:text];
            break;
        case FORMAT_MONEY_INT_DOL:
            returnString = [self toMoneyIntDol:text];
            break;
        case FORMAT_MONEY_DEC_DOL:
            returnString = [self toMoneyDecDol:text];
            break;
        default:
            break;
    }
    
    return returnString;
}

/**
 半角变全角
 @param text
 @returns 全角
 */
- (NSString*)commaToFull:(NSString*)text{
    NSString* returnText = [text stringByReplacingOccurrencesOfString:@"," withString:@"，"];
    returnText = [returnText stringByReplacingOccurrencesOfString:@"'" withString:@"’"];
    returnText = [returnText stringByReplacingOccurrencesOfString:@"\"" withString:@"”"];
    returnText = [returnText stringByReplacingOccurrencesOfString:@"\\" withString:@"￥"];
    
    return returnText;
}

/**
 数值加逗号
 @param text
 @returns
 */
- (NSString*)addComma:(NSString*)text
{
    /** 整数部 */
    NSNumber *number = [[NSNumber alloc] initWithInt:[text intValue]];
    NSNumberFormatter *fmt = [[NSNumberFormatter alloc] init];
    [fmt setNumberStyle:NSNumberFormatterDecimalStyle];
    [fmt setGroupingSeparator:@","];
    [fmt setGroupingSize:3];
    
    /** 小数部 */
    if ([self matchPattern:text chekingType:CheckTypeCombileNo] == TEXTTYPE_DECIMAL ||
        [self matchPattern:text chekingType:CheckTypeCombileNo] == TEXTTYPE_NUMBER_DEC
        ) {
        
        while([text hasSuffix:@"0"]){
            text = [text substringToIndex:text.length-1];
        }
        if([text hasSuffix:@"."]){
            //text = [text substringToIndex:text.length-1];
            
        } else {
            NSInteger index = [text rangeOfString:@"."].location;
            
            text = [NSString stringWithFormat:@"%@.%@",
                    [fmt stringForObjectValue:number],[text substringFromIndex:index+1]];
            return text;
        }
    }
    
    text = [fmt stringForObjectValue:number];
    return text;
}

/**
 文字半角转全角
 @param text
 @returns
 */
- (NSString*) toFull:(NSString*)text
{
    return [self stringTransformWithTransform:kCFStringTransformFullwidthHalfwidth
                                      reverse:true text:text];
}

/**
 文字列全角转半角
 @param text
 @returns
 */
- (NSString*) toHalf:(NSString*)text {
    return [self stringTransformWithTransform:kCFStringTransformFullwidthHalfwidth
                                      reverse:false text:text];
}

/** 全角ー半角 */
- (NSString*) stringTransformWithTransform:(CFStringRef)transform
                                   reverse:(Boolean)reverse text:(NSString*)text{
    NSMutableString* retStr = [[NSMutableString alloc] initWithString:text];
    CFStringTransform((CFMutableStringRef)retStr, NULL, transform, reverse);
    return retStr;
}




/**
 yyyyMMdd根据斜杠切割
 @param text
 @returns
 */
- (NSString*)dateAddSlush:(NSString*)text
{
    NSString* returnString = text;
    NSRange range;
    
    //　yyyyMMddのとき
    range = [text rangeOfString:@"[0-9]{8}" options:NSRegularExpressionSearch];
    if ( [text length] == range.length ) {
        NSString *str1 = [text substringToIndex:4];
        NSString *str2 = [text substringWithRange:NSMakeRange(4,2)];
        NSString *str3 = [text substringWithRange:NSMakeRange(6,2)];
        returnString = [NSString stringWithFormat:@"%@/%@/%@",str1,str2,str3];
    }
    
    //　yyyy-MM(0M,M)-dd(0d,d)のとき
    range = [text rangeOfString:@"[0-9]{4}-[0-9]{1,2}-[0-9]{1,2}" options:NSRegularExpressionSearch];
    if ( [text length] == range.length ) {
        NSArray* array = [text componentsSeparatedByString:@"-"];
        NSString *str1 = [array objectAtIndex:0];
        NSString *str2 = [array objectAtIndex:1];
        NSString *str3 = [array objectAtIndex:2];
        if(str2.length <= 1){
            str2 = [NSString stringWithFormat:@"0%@",str2];
        }
        if(str3.length <= 1){
            str3 = [NSString stringWithFormat:@"0%@",str3];
        }
        returnString = [NSString stringWithFormat:@"%@-%@-%@",str1,str2,str3];
    }
    
    //　yyyy/MM/ddのとき
    range = [text rangeOfString:@"[0-9]{4}/[0-9]{1,2}/[0-9]{1,2}" options:NSRegularExpressionSearch];
    if ( [text length] == range.length ) {
        NSArray* array = [text componentsSeparatedByString:@"/"];
        NSString *str1 = [array objectAtIndex:0];
        NSString *str2 = [array objectAtIndex:1];
        if(str2.length <= 1){
            str2 = [NSString stringWithFormat:@"0%@",str2];
        }
        NSString *str3 = [array objectAtIndex:2];
        if(str3.length <= 1){
            str3 = [NSString stringWithFormat:@"0%@",str3];
        }
        returnString = [NSString stringWithFormat:@"%@/%@/%@",str1,str2,str3];
    }
    
    //　MM-dd,MM-d,M-dd.M-dのとき
    range = [text rangeOfString:@"[0-9]{1,2}-[0-9]{1,2}" options:NSRegularExpressionSearch];
    if ( [text length] == range.length ) {
        NSDate *date = [NSDate date];
        NSCalendar *calendar = [NSCalendar currentCalendar];
        NSDateComponents *comps;
        
        // 年月日をとりだす
        comps = [calendar components:(NSCalendarUnitYear | NSCalendarUnitMonth)
                            fromDate:date];
        NSInteger year = [comps year];
        NSInteger month = [comps month];
        
        if(month == 1){
            year--;
        }
        
        NSRange range2 = [text rangeOfString:@"-" options:NSRegularExpressionSearch];
        
        if(range2.location != INT32_MAX ){
            // @"-"が存在する
            if(range.length == 5){
                // MM-dd
                returnString = [NSString stringWithFormat:@"%@-%@",[self strValue:year],text];
            }
            if(range.length == 4){
                if(range2.location == 1){
                    // M-dd
                    returnString = [NSString stringWithFormat:@"%@-0%@-%@",[self strValue:year],
                                    [text substringToIndex:1],
                                    [text substringFromIndex:2]];
                }
                if(range2.location == 2){
                    // MM-d
                    returnString = [NSString stringWithFormat:@"%@-%@-0%@",[self strValue:year],
                                    [text substringToIndex:2],
                                    [text substringFromIndex:3]];
                }
            }
            if(range.length == 3){
                // M-d
                returnString = [NSString stringWithFormat:@"%@-0%@-0%@",[self strValue:year],
                                [text substringToIndex:1],
                                [text substringFromIndex:2]];
            }
        } else {
            // @"-"が存在しない
            
            // MMddのとき
            if(range.length == 4) {
                returnString = [NSString stringWithFormat:@"%@-%@-%@",[self strValue:year],
                                [text substringToIndex:2],
                                [text substringFromIndex:2]];
            }
            
        }
    }
    
    //　MM-dd,MM-d,M-dd.M-dのとき
    range = [text rangeOfString:@"[0-9]{1,2}-[0-9]{1,2}" options:NSRegularExpressionSearch];
    if ( [text length] == range.length ) {
        NSDate *date = [NSDate date];
        NSCalendar *calendar = [NSCalendar currentCalendar];
        NSDateComponents *comps;
        
        // 年月日をとりだす
        comps = [calendar components:(NSCalendarUnitYear | NSCalendarUnitMonth)
                            fromDate:date];
        NSInteger year = [comps year];
        NSInteger month = [comps month];
        
        if(month == 1){
            year--;
        }
        
        NSRange range2 = [text rangeOfString:@"-" options:NSRegularExpressionSearch];
        
        if(range2.location != INT32_MAX ){
            // @"-"が存在する
            if(range.length == 5){
                // MM-dd
                returnString = [NSString stringWithFormat:@"%@-%@",[self strValue:year],text];
            }
            if(range.length == 4){
                if(range2.location == 1){
                    // M-dd
                    returnString = [NSString stringWithFormat:@"%@-0%@-%@",[self strValue:year],
                                    [text substringToIndex:1],
                                    [text substringFromIndex:2]];
                }
                if(range2.location == 2){
                    // MM-d
                    returnString = [NSString stringWithFormat:@"%@-%@-0%@",[self strValue:year],
                                    [text substringToIndex:2],
                                    [text substringFromIndex:3]];
                }
            }
            if(range.length == 3){
                // M-d
                returnString = [NSString stringWithFormat:@"%@-0%@-0%@",[self strValue:year],
                                [text substringToIndex:1],
                                [text substringFromIndex:2]];
            }
        }
    }
    
    //　MM/dd,MM/d,M/dd.M/d,MMddのとき
    // MMddは結局yyyy/MM/ddとなる
    range = [text rangeOfString:@"[0-9]{1,2}/?[0-9]{1,2}" options:NSRegularExpressionSearch];
    if ( [text length] == range.length ) {
        NSDate *date = [NSDate date];
        NSCalendar *calendar = [NSCalendar currentCalendar];
        NSDateComponents *comps;
        
        // 年月日をとりだす
        comps = [calendar components:(NSCalendarUnitYear | NSCalendarUnitMonth)
                            fromDate:date];
        NSInteger year = [comps year];
        NSInteger month = [comps month];
        
        if(month == 1){
            year--;
        }
        
        NSRange range2 = [text rangeOfString:@"/" options:NSRegularExpressionSearch];
        
        if(range2.location != INT32_MAX ){
            // @"/"が存在する
            if(range.length == 5){
                // MM/dd
                returnString = [NSString stringWithFormat:@"%@/%@",[self strValue:year],text];
            }
            if(range.length == 4){
                if(range2.location == 1){
                    // M/dd
                    returnString = [NSString stringWithFormat:@"%@/0%@/%@",[self strValue:year],
                                    [text substringToIndex:1],
                                    [text substringFromIndex:2]];
                }
                if(range2.location == 2){
                    // MM/d
                    returnString = [NSString stringWithFormat:@"%@/%@/0%@",[self strValue:year],
                                    [text substringToIndex:2],
                                    [text substringFromIndex:3]];
                }
            }
            if(range.length == 3){
                // M/d
                returnString = [NSString stringWithFormat:@"%@/0%@/0%@",[self strValue:year],
                                [text substringToIndex:1],
                                [text substringFromIndex:2]];
            }
        } else {
            // @"/"が存在しない
            
            // MMddのとき
            if(range.length == 4) {
                returnString = [NSString stringWithFormat:@"%@/%@/%@",[self strValue:year],
                                [text substringToIndex:2],
                                [text substringFromIndex:2]];
            }
            
        }
        
    }
    
    return returnString;
}


/**
 yyyyMMdd,yyyy/MM/dd,yyyy-MM-dd变成yyyy年MM月dd日
 @param text
 @returns
 */
- (NSString*)dateAddKanji:(NSString*)text
{
    NSString* returnString = text;
    NSRange range;
    
    //　yyyyMMdd
    range = [text rangeOfString:@"[0-9]{8}" options:NSRegularExpressionSearch];
    if ( [text length] == range.length ) {
        NSString *str1 = [text substringToIndex:4];
        NSString *str2 = [text substringWithRange:NSMakeRange(4,2)];
        NSString *str3 = [text substringWithRange:NSMakeRange(6,2)];
        returnString = [NSString stringWithFormat:@"%@年%@月%@日",str1,str2,str3];
    }
    
    //　yyyy-MM-dd
    range = [text rangeOfString:@"[0-9]{4}-[0-9]{1,2}-[0-9]{1,2}" options:NSRegularExpressionSearch];
    if ( [text length] == range.length ) {
        NSArray* array = [text componentsSeparatedByString:@"-"];
        NSString *str1 = [array objectAtIndex:0];
        NSString *str2 = [array objectAtIndex:1];
        NSString *str3 = [array objectAtIndex:2];
        if(str2.length <= 1){
            str2 = [NSString stringWithFormat:@"0%@",str2];
        }
        if(str3.length <= 1){
            str3 = [NSString stringWithFormat:@"0%@",str3];
        }
        returnString = [NSString stringWithFormat:@"%@年%@月%@日",str1,str2,str3];
    }
    
    //　yyyy/MM/dd
    range = [text rangeOfString:@"[0-9]{4}/[0-9]{1,2}/[0-9]{1,2}" options:NSRegularExpressionSearch];
    if ( [text length] == range.length ) {
        NSArray* array = [text componentsSeparatedByString:@"/"];
        NSString *str1 = [array objectAtIndex:0];
        NSString *str2 = [array objectAtIndex:1];
        if(str2.length <= 1){
            str2 = [NSString stringWithFormat:@"0%@",str2];
        }
        NSString *str3 = [array objectAtIndex:2];
        if(str3.length <= 1){
            str3 = [NSString stringWithFormat:@"0%@",str3];
        }
        returnString = [NSString stringWithFormat:@"%@年%@月%@日",str1,str2,str3];
    }
    
    //　MM-dd,MM-d,M-dd.M-d
    range = [text rangeOfString:@"[0-9]{1,2}-[0-9]{1,2}" options:NSRegularExpressionSearch];
    if ( [text length] == range.length ) {
        NSDate *date = [NSDate date];
        NSCalendar *calendar = [NSCalendar currentCalendar];
        NSDateComponents *comps;
        
        // 取出年月日
        comps = [calendar components:(NSCalendarUnitYear | NSCalendarUnitMonth)
                            fromDate:date];
        NSInteger year = [comps year];
        NSInteger month = [comps month];
        
        if(month == 1){
            year--;
        }
        
        NSRange range2 = [text rangeOfString:@"-" options:NSRegularExpressionSearch];
        
        if(range2.location != INT32_MAX ){
            // @"-"存在
            NSArray* array = [text componentsSeparatedByString:@"-"];
            NSString *mm = [array objectAtIndex:0];
            NSString *dd = [array objectAtIndex:1];
            
            if(range.length == 5){
                // MM-dd
                returnString = [NSString stringWithFormat:@"%@年%@月%@日",[self strValue:year],mm,dd];
            }
            if(range.length == 4){
                if(range2.location == 1){
                    // M-dd
                    returnString = [NSString stringWithFormat:@"%@年0%@月%@日",[self strValue:year],
                                    mm,
                                    dd];
                }
                if(range2.location == 2){
                    // MM-d
                    returnString = [NSString stringWithFormat:@"%@年%@月0%@日",[self strValue:year],
                                    mm,
                                    dd];
                }
            }
            if(range.length == 3){
                // M-d
                returnString = [NSString stringWithFormat:@"%@年0%@月0%@日",[self strValue:year],
                                mm,
                                dd];
            }
        }
    }
    
    //　MM/dd,MM/d,M/dd.M/d,MMdd
    // MMdd先转化为yyyy/MM/dd
    range = [text rangeOfString:@"[0-9]{1,2}/?[0-9]{1,2}" options:NSRegularExpressionSearch];
    if ( [text length] == range.length ) {
        NSDate *date = [NSDate date];
        NSCalendar *calendar = [NSCalendar currentCalendar];
        NSDateComponents *comps;
        
        // 取出年月日
        comps = [calendar components:(NSCalendarUnitYear | NSCalendarUnitMonth)
                            fromDate:date];
        NSInteger year = [comps year];
        NSInteger month = [comps month];
        
        if(month == 1){
            year--;
        }
        
        NSRange range2 = [text rangeOfString:@"/" options:NSRegularExpressionSearch];
        
        if(range2.location != INT32_MAX ){
            // 存在@"/"
            NSArray* array = [text componentsSeparatedByString:@"/"];
            NSString *mm = [array objectAtIndex:0];
            NSString *dd = [array objectAtIndex:1];
            if(range.length == 5){
                // MM/dd
                returnString = [NSString stringWithFormat:@"%@年%@月%@日",[self strValue:year],mm,dd];
            }
            if(range.length == 4){
                if(range2.location == 1){
                    // M/dd
                    returnString = [NSString stringWithFormat:@"%@年0%@月%@日",[self strValue:year],
                                    mm,
                                    dd];
                }
                if(range2.location == 2){
                    // MM/d
                    returnString = [NSString stringWithFormat:@"%@年%@月0%@日",[self strValue:year],
                                    mm,
                                    dd];
                }
            }
            if(range.length == 3){
                // M/d
                returnString = [NSString stringWithFormat:@"%@年0%@月0%@日",[self strValue:year],
                                mm,
                                dd];
            }
        } else {
            // 不存在@"/"
            
            // MMdd
            if(range.length == 4) {
                returnString = [NSString stringWithFormat:@"%@年%@月%@日",[self strValue:year],
                                [text substringToIndex:2],
                                [text substringFromIndex:2]];
            }
            
        }
    }
    
    
    return returnString;
}

/**
 添加%记号
 @param text
 @returns
 */
- (NSString*)addPercent:(NSString*)text

{
    NSString* returnString = text;
    
    
    switch ([self matchPattern:text chekingType:CheckTypeCombileNo]) {
        case TEXTTYPE_INTEGER:
        case TEXTTYPE_NUMBER_INT:
            returnString = [NSString stringWithFormat:@"%d%%",[text intValue]];
            break;
            
        case TEXTTYPE_DECIMAL:
        case TEXTTYPE_NUMBER_DEC:{
            
            while(((text.length-1) - [text rangeOfString:@"."].location) > 2){
                text = [text substringToIndex:text.length-1];
            }
            while([text hasSuffix:@"0"]){
                text = [text substringToIndex:text.length-1];
            }
            if([text hasSuffix:@"."]){
                text = [text substringToIndex:text.length-1];
                returnString = [NSString stringWithFormat:@"%d%%",[text intValue]];
            } else {
                NSInteger index = [text rangeOfString:@"."].location;
                
                returnString = [NSString stringWithFormat:@"%d.%@%%",
                                [text intValue],[text substringFromIndex:index+1]];
            }
        }
            break;
            
        default:
            break;
    }
    
    return returnString;
}

/**
 ¥货币格式、舍去小数点
 @param text
 @returns
 */
- (NSString*)toMoneyIntYen:(NSString*)text

{
    text = [NSString stringWithFormat:@"%d",[text intValue]];
    text = [self addComma:text];
    
    return [NSString stringWithFormat:@"¥%@",text];
}

- (NSString*)toMoneyDecYen:(NSString*)text{
    NSString* returnString = text;
    
    switch ([self matchPattern:text chekingType:CheckTypeCombileNo]) {
        case TEXTTYPE_INTEGER:
        case TEXTTYPE_NUMBER_INT:
            text = [NSString stringWithFormat:@"%d",[text intValue]];
            text = [self addComma:text];
            returnString = [NSString stringWithFormat:@"¥%@",text];
            break;
            
        case TEXTTYPE_DECIMAL:
        case TEXTTYPE_NUMBER_DEC:{
            
            while(((text.length-1) - [text rangeOfString:@"."].location) > 2){
                text = [text substringToIndex:text.length-1];
            }
            while([text hasSuffix:@"0"]){
                text = [text substringToIndex:text.length-1];
            }
            if([text hasSuffix:@"."]){
                text = [text substringToIndex:text.length-1];
                text = [NSString stringWithFormat:@"%d",[text intValue]];
                text = [self addComma:text];
                returnString = [NSString stringWithFormat:@"¥%@",text];
            } else {
                NSInteger index = [text rangeOfString:@"."].location;
                
                NSString* decStr = [text substringFromIndex:index+1];
                text = [NSString stringWithFormat:@"%d",[text intValue]];
                text = [self addComma:text];
                returnString = [NSString stringWithFormat:@"¥%@.%@",text,decStr];
            }
        }
            break;
            
        default:
            break;
    }
    
    return returnString;
}

- (NSString*)toMoneyIntDol:(NSString*)text{
    text = [NSString stringWithFormat:@"%ds",[text intValue]];
    text = [self addComma:text];
    
    return [NSString stringWithFormat:@"$%@",text];
}

- (NSString*)toMoneyDecDol:(NSString*)text{
    NSString* returnString = text;
    
    switch ([self matchPattern:text chekingType:CheckTypeCombileNo]) {
        case TEXTTYPE_INTEGER:
        case TEXTTYPE_NUMBER_INT:
            text = [NSString stringWithFormat:@"%d",[text intValue]];
            text = [self addComma:text];
            returnString = [NSString stringWithFormat:@"$%@",text];
            break;
            
        case TEXTTYPE_DECIMAL:
        case TEXTTYPE_NUMBER_DEC:{
            
            while(((text.length-1) - [text rangeOfString:@"."].location) > 2){
                text = [text substringToIndex:text.length-1];
            }
            while([text hasSuffix:@"0"]){
                text = [text substringToIndex:text.length-1];
            }
            if([text hasSuffix:@"."]){
                text = [text substringToIndex:text.length-1];
                text = [NSString stringWithFormat:@"%d",[text intValue]];
                text = [self addComma:text];
                returnString = [NSString stringWithFormat:@"$%@",text];
            } else {
                NSInteger index = [text rangeOfString:@"."].location;
                
                NSString* decStr = [text substringFromIndex:index+1];
                text = [NSString stringWithFormat:@"%d",[text intValue]];
                text = [self addComma:text];
                returnString =
                [NSString stringWithFormat:@"$%@.%@",text,decStr];
            }
        }
            break;
            
        default:
            break;
    }
    
    return returnString;
}


/*****************************************************************************
 输入辅助
 @param text textfield里现有的值
 @param deformatType 转换前的类型
 @returns 转换前的文字
 */
- (NSString*)deformat:(NSString*)text deformatType:(int)type

{
    NSString* returnString = text;
    
    switch (type) {
        case FORMAT_NUM_COMMA:
            returnString = [self removeComma:text];
            break;
            
        case FORMAT_MONEY_INT_YEN:
        case FORMAT_MONEY_DEC_YEN:
        case FORMAT_MONEY_INT_DOL:
        case FORMAT_MONEY_DEC_DOL:
            returnString = [self removeMoney:text];
            break;
            
        case FORMAT_RATIO:
            returnString = [self removePercent:text];
            break;
            
        case FORMAT_DATE_KANJI:
            returnString = [self dateKanjiToSlush:text];
            break;
            
        default:
            break;
    }
    return returnString;
}

/**
 去掉逗号
 */
- (NSString*)removeComma:(NSString*)text

{
    return [text stringByReplacingOccurrencesOfString:@"," withString:@""];
}

/**
 货币格式转成普通格式
 */
- (NSString*)removeMoney:(NSString*)text
{
    text = [self removeComma:text];
    text = [text stringByReplacingOccurrencesOfString:@"¥" withString:@""];
    text = [text stringByReplacingOccurrencesOfString:@"$" withString:@""];
    return text;
}

/**
 ％格式转成普通格式
 */
- (NSString*)removePercent:(NSString*)text
{
    return [text stringByReplacingOccurrencesOfString:@"%" withString:@""];
}

/**
 yyyy年MM月dd日格式转成yyyy/MM/dd
 */
- (NSString*)dateKanjiToSlush:(NSString*)text
{
    text = [text stringByReplacingOccurrencesOfString:@"日" withString:@""];
    text = [text stringByReplacingOccurrencesOfString:@"月" withString:@"/"];
    return [text stringByReplacingOccurrencesOfString:@"年" withString:@"/"];
}

/**
 日期格式转成普通格式
 */
- (NSString*)removeDate:(NSString*)text
{
    text = [text stringByReplacingOccurrencesOfString:@"日" withString:@""];
    text = [text stringByReplacingOccurrencesOfString:@"月" withString:@""];
    text = [text stringByReplacingOccurrencesOfString:@"年" withString:@""];
    text = [text stringByReplacingOccurrencesOfString:@"/" withString:@""];
    text = [text stringByReplacingOccurrencesOfString:@"-" withString:@""];
    return text;
}

/*****************************************************************************
 判断是否是正常的值
 */
- (BOOL)isNormalValue:(UITextField*)tf formatType:(int)type
{
    switch (type) {
//        case FORMAT_MASTER:
//            //判断是否和既定格式一致
//            if([self matchMaster:tf.text]){
//                tf.textColor = okColor;
//                return YES;
//            } else {
//                tf.textColor = ngColor;
//                tf.text = @"";
//                return NO;
//            }
//            
//            break;
            
        case FORMAT_RATIO:
        case FORMAT_MONEY_INT_YEN:
        case FORMAT_MONEY_DEC_YEN:
        case FORMAT_MONEY_INT_DOL:
        case FORMAT_MONEY_DEC_DOL:
        case FORMAT_NUM_COMMA:
        {
            NSNumber* num = [NSNumber numberWithFloat:[tf.text floatValue]];
            if([self innerOfRange:[tf.text floatValue]] && ![_ngArray containsObject:num]){
                tf.textColor = okColor;
                return YES;
            } else {
                tf.textColor = ngColor;
                tf.text = @"";
                return NO;
            }
        }
            
            break;
            
        default:
            break;
    }
    
    return YES;
}


/**
 判断是否存在某一个数据
 @param key
 @returns
 */
- (BOOL)matchMaster:(NSString*)key
{
    // ダミー、プロパティリストと値を照合する処理
    NSDictionary* items = [self setItems];
    if([items objectForKey:key] == nil){
        return NO;
    } else {
        return YES;
    }
}

- (NSDictionary*)setItems{
    
    NSDictionary* items;
    NSBundle* bundle;
    NSString* file;
    
    bundle = [NSBundle mainBundle];
    file = [bundle pathForResource:@"ITEMS" ofType:@"plist"];
    
    if (file){
        items = [NSDictionary dictionaryWithContentsOfFile:file];
    } else {
        items = nil;
    }
    
    return items;
}



/**
 指定数字型数据范围
 @param min 最小値
 @param max 最大値
 */
- (void)setValueRangeMinToMax:(NSNumber*)min max:(NSNumber*)max
{
    _min = [min copy];
    _max = [max copy];
}

/**
 指定变成ng的数组

 @param ngArray 。内容是NSNumber
 */
- (void)setNgValue:(NSArray*)ngArray
{
    _ngArray = ngArray;
}

/**
 判断数值是否在指定范围内
 @param value
 @returns    YES:有
 NO:没有
 */
- (BOOL)innerOfRange:(float)value

{
    if([_min floatValue] <= value && value <= [_max floatValue]){
        return YES;
    } else {
        return NO;
    }
}


/**
判断是否是有效的日期
 @param tf 输入框
 @returns YES: NO:
 */
- (BOOL)isNormalDate:(UITextField*)tf
{
    if(formatType == FORMAT_DATE_SLUSH ||
       formatType == FORMAT_DATE_KANJI){
        PUT_LOG(@"%@ isNormalDate tf.text = %@",CLASS_NAME,tf.text);
        NSString* str = tf.text;
        //        NSString* str = [self removeDate:tf.text];
        //        str = [self dateAddSlush:str];
        PUT_LOG(@"%@ isNormalDate str = %@",CLASS_NAME,str);
        NSDateFormatter* formatter = [[NSDateFormatter alloc] init];
        [formatter setDateFormat:@"yyyy/MM/dd"];
        [formatter setTimeZone:[NSTimeZone timeZoneForSecondsFromGMT:0]];
        NSDate* slushDate = [formatter dateFromString:str];
        
        [formatter setDateFormat:@"yyyy-MM-dd"];
        NSDate* hyphenDate = [formatter dateFromString:str];
        
        [formatter setDateFormat:@"yyyy年MM月dd日"];
        NSDate* kanjiDate = [formatter dateFromString:str];
        
        PUT_LOG(@"%@ isNormalDate date = %@",CLASS_NAME,slushDate);
        
        // 正常な日時でなければnil
        if(slushDate != nil &&
           earlyDay == [earlyDay earlierDate:slushDate] &&
           laterDay == [laterDay laterDate:slushDate]){
            tf.textColor = okColor;
            return YES;
        } else if(hyphenDate != nil &&
                  earlyDay == [earlyDay earlierDate:hyphenDate] &&
                  laterDay == [laterDay laterDate:hyphenDate]){
            tf.textColor = okColor;
            return YES;
        } else if(kanjiDate != nil &&
                  earlyDay == [earlyDay earlierDate:kanjiDate] &&
                  laterDay == [laterDay laterDate:kanjiDate]){
            tf.textColor = okColor;
            return YES;
        } else {
            tf.textColor = ngColor;
            tf.text = @"";
            return NO;
        }
    }
    return YES;
}


/**
 指定日期范围
 @param early    开始日期
 @param late     结束日期
 */
- (void)setDateRangeWithEarly:(NSDate*)early late:(NSDate*)late

{
    earlyDay = [early copy];
    laterDay = [late copy];
}

/*****************************************************************************/
// UITextFieldDelegate

- (BOOL)textField:(UITextField *)textField shouldChangeCharactersInRange:(NSRange)range replacementString:(NSString *)string{
    

    
    
    PUT_LOG_METHOD;
    PUT_LOG(@"%@,shouldChangeCharactersInRange replacementString = %@",CLASS_NAME,string);
    NSMutableString* mutableString = [textField.text mutableCopy];
    [mutableString appendString:string];
    if (self.maxlength != 0 && mutableString.length > self.maxlength) {
        textField.text = [mutableString substringToIndex:self.maxlength];
        return NO;
    }
    if (checkingType != TEXTTYPE_NONE) {
        if([string isEqualToString:@","] ||
           [string isEqualToString:@"'"] ||
           [string isEqualToString:@"\""]){
            return NO;
        }
    }
    
    NSString* newString = [NSString stringWithFormat:@"%@%@",textField.text,string];
    //PUT_LOG(@"%@",string);
    //PUT_LOG(@" checkingType = %d",[self checkLimitation:newString limitationType:checkingType]);
    return [self checkLimitation:newString limitationType:checkingType];
}

-(void)textFieldDidEndEditing:(UITextField*)textField{
    [self isNormalValue:textField formatType:formatType];
    textField.text = [self format:textField.text formatType:formatType];
    [self isNormalDate:textField];

}


-(BOOL)textFieldShouldBeginEditing:(UITextField*)textField{
    //textField.textColor = [UIColor blackColor];
    textField.text = [self deformat:textField.text deformatType:formatType];
    
    return YES;
}

-(BOOL)textFieldShouldReturn:(UITextField *)textField{
    [textField resignFirstResponder];
    return YES;
}


- (NSString *)strValue:(NSInteger)value{
#if __LP64__ || (TARGET_OS_EMBEDDED && !TARGET_OS_IPHONE) || TARGET_OS_WIN32 || NS_BUILD_32_LIKE_64
    return [NSString stringWithFormat:@"%ld",value];
#else
    return [NSString stringWithFormat:@"%d",value];
#endif
}

- (void)textFiledEditChanged:(NSNotification*)obj{
    
    UITextField *textfiled = (UITextField *)obj.object;
    
    NSString *toBeString = textfiled.text;
    NSString *lang = [[[UIApplication sharedApplication]textInputMode] primaryLanguage]; // 键盘输入模式
    if ([lang isEqualToString:@"zh-Hans"]) { // 简体中文输入，包括简体拼音，健体五笔，简体手写
        UITextRange *selectedRange = [textfiled markedTextRange];
        //获取高亮部分
        UITextPosition *position = [textfiled positionFromPosition:selectedRange.start offset:0];
        // 没有高亮选择的字，则对已输入的文字进行字数统计和限制
        if (!position) {
            if (toBeString.length > self.maxlength) {
                textfiled.text = [toBeString substringToIndex:maxlength];
            }
        }
        // 有高亮选择的字符串，则暂不对文字进行统计和限制
        else{
            
        }
    }
    // 中文输入法以外的直接对其统计限制即可，不考虑其他语种情况
//    else{
//        if (toBeString.length > maxlength) {
//            textView.text = [toBeString substringToIndex:maxlength];
//        }
//    }
}

@end
